@@ -6,11 +6,11 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000802
+#define IP_VS_VERSION_CODE            0x000900
 #define NVERSION(version)                       \
-        (version >> 16) & 0xFF,                 \
-        (version >> 8) & 0xFF,                  \
-        version & 0xFF                       
+	(version >> 16) & 0xFF,                 \
+	(version >> 8) & 0xFF,                  \
+	version & 0xFF
 
 /*
  *      Virtual Service Flags
@@ -38,21 +38,21 @@
 #define IP_VS_SCHEDNAME_MAXLEN         16
 
 struct ip_vs_rule_user {
-        /* global options */
-        int             tcp_timeout;    /* timeout values */
-        int             tcp_fin_timeout;
-        int             udp_timeout;
-        
+	/* global options */
+	int             tcp_timeout;    /* timeout values */
+	int             tcp_fin_timeout;
+	int             udp_timeout;
+
 	/* virtual service options */
 	u_int16_t	protocol;
 	u_int32_t	vaddr;          /* virtual address */
 	u_int16_t	vport;
-        u_int32_t       vfwmark;        /* firwall mark of virtual */
-        char            sched_name[IP_VS_SCHEDNAME_MAXLEN];
+	u_int32_t       vfwmark;        /* firwall mark of virtual */
+	char            sched_name[IP_VS_SCHEDNAME_MAXLEN];
 	unsigned	vs_flags;       /* virtual service flags */
 	unsigned        timeout;        /* persistent timeout in ticks */
 	u_int32_t	netmask;        /* persistent netmask */
-        
+
 	/* destination specific options */
 	u_int32_t	daddr;          /* real destination address */
 	u_int16_t	dport;
@@ -64,7 +64,7 @@ struct ip_vs_rule_user {
 /*
  *      IPVS socket options
  */
-#define IP_VS_BASE_CTL  	(64+1024+64)            /* base */
+#define IP_VS_BASE_CTL	(64+1024+64)            /* base */
 
 #define IP_VS_SO_SET_NONE	IP_VS_BASE_CTL	        /* just peek */
 #define IP_VS_SO_SET_INSERT	(IP_VS_BASE_CTL+1)
@@ -91,15 +91,14 @@ struct ip_vs_rule_user {
 #define IP_VS_CONN_F_LOCALNODE	      0x0001    /* local node */
 #define IP_VS_CONN_F_TUNNEL	      0x0002    /* tunneling */
 #define IP_VS_CONN_F_DROUTE           0x0003    /* direct routing */
-#define IP_VS_CONN_F_HASHED	      0x0040 	/* hashed entry */
+#define IP_VS_CONN_F_HASHED	      0x0040	/* hashed entry */
 #define IP_VS_CONN_F_NOOUTPUT         0x0080    /* no output packets */
 #define IP_VS_CONN_F_INACTIVE         0x0100    /* not established */
 #define IP_VS_CONN_F_OUT_SEQ          0x0200    /* must do output seq adjust */
 #define IP_VS_CONN_F_IN_SEQ           0x0400    /* must do input seq adjust */
 #define IP_VS_CONN_F_NO_CPORT         0x0800    /* no client port set yet */
 
-/* Move it to better place one day, for now keep it unique */
-#define NFC_IPVS_PROPERTY	0x10000
+#define NFMARK_IPVS_PROPERTY	      -0x100
 
 #ifdef __KERNEL__
 
@@ -111,41 +110,38 @@ struct ip_vs_rule_user {
 #include <asm/atomic.h>                 /* for struct atomic_t */
 #include <linux/netdevice.h>		/* for struct neighbour; */
 #include <net/dst.h>			/* for struct dst_entry */
-#include <linux/in6.h>
-#include <asm/uaccess.h>
-#include <asm/checksum.h>		/* for csum_partial */
 
 
 /*  #define CONFIG_IP_VS_DEBUG */
 
 #ifdef CONFIG_IP_VS_DEBUG
 extern int ip_vs_get_debug_level(void);
-#define IP_VS_DBG(level, msg...)			\
-    do {						\
-            if (level <= ip_vs_get_debug_level())	\
-                    printk(KERN_DEBUG "IPVS: " msg);	\
+#define IP_VS_DBG(level, msg...)                        \
+    do {                                                \
+	    if (level <= ip_vs_get_debug_level())       \
+		    printk(KERN_DEBUG "IPVS: " ## msg); \
     } while (0)
 #else	/* NO DEBUGGING at ALL */
 #define IP_VS_DBG(level, msg...)  do {} while (0)
 #endif
 
-#define IP_VS_ERR(msg...) printk(KERN_ERR "IPVS: " msg)
-#define IP_VS_INFO(msg...) printk(KERN_INFO "IPVS: " msg)
+#define IP_VS_ERR(msg...) printk(KERN_ERR "IPVS: " ## msg )
+#define IP_VS_INFO(msg...) printk(KERN_INFO "IPVS: " ## msg )
 #define IP_VS_WARNING(msg...) \
-	printk(KERN_WARNING "IPVS: " msg)
+	printk(KERN_WARNING "IPVS: " ## msg)
 
 #ifdef CONFIG_IP_VS_DEBUG
 #define EnterFunction(level)                                            \
     do {                                                                \
-            if (level <= ip_vs_get_debug_level())                       \
-                    printk(KERN_DEBUG "Enter: %s, %s line %i\n",        \
-                           __FUNCTION__, __FILE__, __LINE__);           \
+	    if (level <= ip_vs_get_debug_level())                       \
+		    printk(KERN_DEBUG "Enter: %s, %s line %i\n",        \
+			   __FUNCTION__, __FILE__, __LINE__);           \
     } while (0)
 #define LeaveFunction(level)                                            \
     do {                                                                \
-            if (level <= ip_vs_get_debug_level())                       \
-                        printk(KERN_DEBUG "Leave: %s, %s line %i\n",    \
-                               __FUNCTION__, __FILE__, __LINE__);       \
+	    if (level <= ip_vs_get_debug_level())                       \
+			printk(KERN_DEBUG "Leave: %s, %s line %i\n",    \
+			       __FUNCTION__, __FILE__, __LINE__);       \
     } while (0)
 #else
 #define EnterFunction(level)   do {} while (0)
@@ -186,6 +182,7 @@ enum {
 	NET_IPV4_VS_TO_ICMP=18,
 	NET_IPV4_VS_LBLC_EXPIRE=19,
 	NET_IPV4_VS_LBLCR_EXPIRE=20,
+	NET_IPV4_VS_USE_DST=21,
 	NET_IPV4_VS_CACHE_BYPASS=22,
 };
 
@@ -218,11 +215,11 @@ struct ip_vs_timeout_table {
 };
 
 union ip_vs_tphdr {
-        unsigned char *raw;
-        struct udphdr *uh;
-        struct tcphdr *th;
-        struct icmphdr *icmph;
-        __u16 *portp;
+	unsigned char *raw;
+	struct udphdr *uh;
+	struct tcphdr *th;
+	struct icmphdr *icmph;
+	__u16 *portp;
 };
 
 
@@ -232,10 +229,10 @@ union ip_vs_tphdr {
  *      Only used in the VS/NAT.
  */
 struct ip_vs_seq {
-        __u32           init_seq;       /* Add delta from this seq */
-        __u32           delta;          /* Delta in sequence numbers */
-        __u32           previous_delta; /* Delta in sequence numbers
-                                           before last resized pkt */
+	__u32           init_seq;       /* Add delta from this seq */
+	__u32           delta;          /* Delta in sequence numbers */
+	__u32           previous_delta; /* Delta in sequence numbers
+					   before last resized pkt */
 };
 
 
@@ -244,12 +241,12 @@ struct ip_vs_seq {
  */
 struct ip_vs_stats
 {
-        spinlock_t              lock;           /* spin lock */
-        __u32                   conns;          /* connections scheduled */
-        __u32                   inpkts;         /* incoming packets */
-        __u32                   outpkts;        /* outgoing packets */
-        __u64                   inbytes;        /* incoming bytes */
-        __u64                   outbytes;       /* outgoing bytes */
+	spinlock_t              lock;           /* spin lock */
+	__u32                   conns;          /* connections scheduled */
+	__u32                   inpkts;         /* incoming packets */
+	__u32                   outpkts;        /* outgoing packets */
+	__u64                   inbytes;        /* incoming bytes */
+	__u64                   outbytes;       /* outgoing bytes */
 };
 
 
@@ -261,38 +258,38 @@ struct ip_vs_conn {
 	atomic_t                refcnt;         /* reference count */
 	struct timer_list       timer;          /* Expiration timer */
 
-        /* Protocol, addresses and port numbers */
-        __u32                   caddr;          /* client address */
-        __u32                   vaddr;          /* virtual address */
-        __u32                   daddr;          /* destination address */
-        __u16                   cport;
-        __u16                   vport;
-        __u16                   dport;
+	/* Protocol, addresses and port numbers */
+	__u32                   caddr;          /* client address */
+	__u32                   vaddr;          /* virtual address */
+	__u32                   daddr;          /* destination address */
+	__u16                   cport;
+	__u16                   vport;
+	__u16                   dport;
 	__u16                   protocol;       /* Which protocol (TCP/UDP) */
 
-        /* Flags and state transition */
-        spinlock_t              lock;           /* lock for state transition */
+	/* Flags and state transition */
+	spinlock_t              lock;           /* lock for state transition */
 	volatile __u16          flags;          /* status flags */
 	volatile __u16          state;          /* state info */
 	volatile unsigned long  timeout;        /* timeout */
-        struct ip_vs_timeout_table *timeout_table;
+	struct ip_vs_timeout_table *timeout_table;
 
 	/* Control members */
-        struct ip_vs_conn       *control;       /* Master control connection */
+	struct ip_vs_conn       *control;       /* Master control connection */
 	atomic_t                n_control;      /* Number of controlled ones */
 	struct ip_vs_dest       *dest;          /* real server */
-        atomic_t                in_pkts;        /* incoming packet counter */
+	atomic_t                in_pkts;        /* incoming packet counter */
 
-        /* packet transmitter for different forwarding methods */
-        int (*packet_xmit)(struct sk_buff *skb, struct ip_vs_conn *cp);
+	/* packet transmitter for different forwarding methods */
+	int (*packet_xmit)(struct sk_buff *skb, struct ip_vs_conn *cp);
 
-        /* Note: we can group the following members into a structure,
-           in order to save more space, and the following members are
-           only used in VS/NAT anyway */
+	/* Note: we can group the following members into a structure,
+	   in order to save more space, and the following members are
+	   only used in VS/NAT anyway */
 	struct ip_vs_app        *app;           /* bound ip_vs_app object */
 	void                    *app_data;      /* Application private data */
-        struct ip_vs_seq        in_seq;         /* incoming seq. struct */
-        struct ip_vs_seq        out_seq;        /* outgoing seq. struct */
+	struct ip_vs_seq        in_seq;         /* incoming seq. struct */
+	struct ip_vs_seq        out_seq;        /* outgoing seq. struct */
 };
 
 
@@ -304,6 +301,8 @@ struct ip_vs_service {
 	struct list_head	s_list;   /* for normal service table */
 	struct list_head	f_list;   /* for fwmark-based service table */
 	atomic_t		refcnt;   /* reference counter */
+	atomic_t		usecnt;   /* use counter */
+
 	__u16			protocol; /* which protocol (TCP/UDP) */
 	__u32			addr;	  /* IP address for virtual service */
 	__u16			port;	  /* port number for the service */
@@ -312,10 +311,12 @@ struct ip_vs_service {
 	unsigned		timeout;  /* persistent timeout in ticks */
 	__u32			netmask;  /* grouping granularity */
 	struct list_head	destinations;  /* real server d-linked list */
+	struct ip_vs_stats      stats;         /* statistics for the service */
+
+	/* for scheduling */
 	struct ip_vs_scheduler	*scheduler;    /* bound scheduler object */
 	rwlock_t		sched_lock;    /* lock sched_data */
 	void			*sched_data;   /* scheduler application data */
-        struct ip_vs_stats      stats;         /* statistics for the service */
 };
 
 
@@ -330,21 +331,21 @@ struct ip_vs_dest {
 	__u32			addr;	  /* IP address of real server */
 	__u16			port;	  /* port number of the service */
 	unsigned		flags;	  /* dest status flags */
-	atomic_t		weight; 	/* server weight */
+	atomic_t		weight;	/* server weight */
 	atomic_t		conn_flags;	/* flags to copy to conn */
 	atomic_t		activeconns;	/* active connections */
 	atomic_t		inactconns;	/* inactive connections */
-	atomic_t		refcnt; 	/* reference counter */
-        struct ip_vs_stats      stats;          /* statistics */
+	atomic_t		refcnt;	/* reference counter */
+	struct ip_vs_stats      stats;          /* statistics */
 
-        /* for destination cache */
+	/* for destination cache */
 	spinlock_t		dst_lock;	/* lock dst_cache */
 	struct dst_entry	*dst_cache;	/* destination cache entry */
 	u32			dst_rtos;	/* RT_TOS(tos) for dst */
 
 	/* for virtual service */
 	struct ip_vs_service    *svc;     /* service that it belongs to */
-        __u16			protocol; /* which protocol (TCP/UDP) */
+	__u16			protocol; /* which protocol (TCP/UDP) */
 	__u32			vaddr;	  /* IP address for virtual service */
 	__u16			vport;	  /* port number for the service */
 	__u32                   vfwmark;  /* firewall mark of the service */
@@ -356,20 +357,20 @@ struct ip_vs_dest {
  */
 struct ip_vs_scheduler {
 	struct list_head        n_list;   /* d-linked list head */
-	char 			*name;    /* scheduler name */
-	atomic_t		refcnt;   /* reference counter */
+	char			*name;    /* scheduler name */
+	atomic_t                refcnt;   /* reference counter */
 	struct module		*module;  /* THIS_MODULE/NULL */
 
-        /* scheduler initializing service */
+	/* scheduler initializing service */
 	int (*init_service)(struct ip_vs_service *svc);
-        /* scheduling service finish */
-        int (*done_service)(struct ip_vs_service *svc);
-        /* scheduler updating service */
-        int (*update_service)(struct ip_vs_service *svc);
+	/* scheduling service finish */
+	int (*done_service)(struct ip_vs_service *svc);
+	/* scheduler updating service */
+	int (*update_service)(struct ip_vs_service *svc);
 
 	/* selecting a server from the given service */
 	struct ip_vs_dest* (*schedule)(struct ip_vs_service *svc,
-                                       struct iphdr *iph);
+				       struct iphdr *iph);
 };
 
 
@@ -378,35 +379,35 @@ struct ip_vs_scheduler {
  */
 struct ip_vs_app
 {
-        struct list_head        n_list;   /* d-linked list head */
-        char                    *name;    /* name of application module */
-        unsigned                type;     /* type = proto<<16 | port
-                                             (host byte order)*/
+	struct list_head        n_list;   /* d-linked list head */
+	char                    *name;    /* name of application module */
+	unsigned                type;     /* type = proto<<16 | port
+					     (host byte order)*/
 	struct module		*module;  /* THIS_MODULE/NULL */
-        
-        /* ip_vs_app initializer */
-        int (*init_conn)(struct ip_vs_app *, struct ip_vs_conn *);
-        /* ip_vs_app finish */
-        int (*done_conn)(struct ip_vs_app *, struct ip_vs_conn *);
-        /* output hook */
-        int (*pkt_out)(struct ip_vs_app *,
-                       struct ip_vs_conn *, struct sk_buff **);
-        /* input hook */
-        int (*pkt_in)(struct ip_vs_app *,
-                      struct ip_vs_conn *, struct sk_buff **);
+
+	/* ip_vs_app initializer */
+	int (*init_conn)(struct ip_vs_app *, struct ip_vs_conn *);
+	/* ip_vs_app finish */
+	int (*done_conn)(struct ip_vs_app *, struct ip_vs_conn *);
+	/* output hook */
+	int (*pkt_out)(struct ip_vs_app *,
+		       struct ip_vs_conn *, struct sk_buff **);
+	/* input hook */
+	int (*pkt_in)(struct ip_vs_app *,
+		      struct ip_vs_conn *, struct sk_buff **);
 };
 
-        
+
 /*
  *      IPVS core functions
  *      (from ip_vs_core.c)
  */
 extern struct ip_vs_conn * ip_vs_schedule(struct ip_vs_service *svc,
-                                          struct iphdr *iph);
+					  struct iphdr *iph);
 extern int ip_vs_proto_doff(unsigned proto, char *th, unsigned size);
 extern const char *ip_vs_proto_name(unsigned proto);
 extern unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
-                                        int (*okfn)(struct sk_buff *));
+					int (*okfn)(struct sk_buff *));
 
 
 /*
@@ -435,14 +436,21 @@ extern struct ip_vs_conn *ip_vs_conn_in_get
 (int protocol, __u32 s_addr, __u16 s_port, __u32 d_addr, __u16 d_port);
 extern struct ip_vs_conn *ip_vs_conn_out_get
 (int protocol, __u32 s_addr, __u16 s_port, __u32 d_addr, __u16 d_port);
-extern void __ip_vs_conn_put(struct ip_vs_conn *cp);
+
+/* put back the conn without restarting its timer */
+static inline void __ip_vs_conn_put(struct ip_vs_conn *cp)
+{
+	atomic_dec(&cp->refcnt);
+}
 extern void ip_vs_conn_put(struct ip_vs_conn *cp);
+
 extern struct ip_vs_conn *
 ip_vs_conn_new(int proto, __u32 maddr, __u16 mport, __u32 saddr,
-               __u16 sport, __u32 daddr, __u16 dport, unsigned flags);
+	       __u16 sport, __u32 daddr, __u16 dport, unsigned flags,
+	       struct ip_vs_dest *dest);
 extern const char * ip_vs_state_name(int state);
 extern int ip_vs_set_state(struct ip_vs_conn *cp, int state_off,
-                           struct iphdr *iph, void *tp);
+			   struct iphdr *iph, void *tp);
 extern int ip_vs_conn_listen(struct ip_vs_conn *cp);
 extern int ip_vs_check_template(struct ip_vs_conn *ct);
 extern void ip_vs_secure_tcp_set(int on);
@@ -458,24 +466,24 @@ static inline void ip_vs_control_del(struct ip_vs_conn *cp)
 	struct ip_vs_conn *ctl_cp = cp->control;
 	if (!ctl_cp) {
 		IP_VS_ERR("request control DEL for uncontrolled: "
-                          "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
-                          NIPQUAD(cp->caddr),ntohs(cp->cport),
-                          NIPQUAD(cp->vaddr),ntohs(cp->vport));
-                return;
+			  "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
+			  NIPQUAD(cp->caddr),ntohs(cp->cport),
+			  NIPQUAD(cp->vaddr),ntohs(cp->vport));
+		return;
 	}
-        
+
 	IP_VS_DBG(7, "DELeting control for: "
-                  "cp.dst=%d.%d.%d.%d:%d ctl_cp.dst=%d.%d.%d.%d:%d\n",
-                  NIPQUAD(cp->caddr),ntohs(cp->cport),
-                  NIPQUAD(ctl_cp->caddr),ntohs(ctl_cp->cport));
-        
+		  "cp.dst=%d.%d.%d.%d:%d ctl_cp.dst=%d.%d.%d.%d:%d\n",
+		  NIPQUAD(cp->caddr),ntohs(cp->cport),
+		  NIPQUAD(ctl_cp->caddr),ntohs(ctl_cp->cport));
+
 	cp->control = NULL;
 	if (atomic_read(&ctl_cp->n_control) == 0) {
 		IP_VS_ERR("BUG control DEL with n=0 : "
-                          "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
-                          NIPQUAD(cp->caddr),ntohs(cp->cport),
-                          NIPQUAD(cp->vaddr),ntohs(cp->vport));
-                return;
+			  "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
+			  NIPQUAD(cp->caddr),ntohs(cp->cport),
+			  NIPQUAD(cp->vaddr),ntohs(cp->vport));
+		return;
 	}
 	atomic_dec(&ctl_cp->n_control);
 }
@@ -485,17 +493,17 @@ ip_vs_control_add(struct ip_vs_conn *cp, struct ip_vs_conn *ctl_cp)
 {
 	if (cp->control) {
 		IP_VS_ERR("request control ADD for already controlled: "
-                          "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
-                          NIPQUAD(cp->caddr),ntohs(cp->cport),
-                          NIPQUAD(cp->vaddr),ntohs(cp->vport));
+			  "%d.%d.%d.%d:%d to %d.%d.%d.%d:%d\n",
+			  NIPQUAD(cp->caddr),ntohs(cp->cport),
+			  NIPQUAD(cp->vaddr),ntohs(cp->vport));
 		ip_vs_control_del(cp);
 	}
-        
+
 	IP_VS_DBG(7, "ADDing control for: "
-                  "cp.dst=%d.%d.%d.%d:%d ctl_cp.dst=%d.%d.%d.%d:%d\n",
-                  NIPQUAD(cp->caddr),ntohs(cp->cport),
-                  NIPQUAD(ctl_cp->caddr),ntohs(ctl_cp->cport));
-        
+		  "cp.dst=%d.%d.%d.%d:%d ctl_cp.dst=%d.%d.%d.%d:%d\n",
+		  NIPQUAD(cp->caddr),ntohs(cp->cport),
+		  NIPQUAD(ctl_cp->caddr),ntohs(ctl_cp->cport));
+
 	cp->control = ctl_cp;
 	atomic_inc(&ctl_cp->n_control);
 }
@@ -507,15 +515,15 @@ ip_vs_control_add(struct ip_vs_conn *cp, struct ip_vs_conn *ctl_cp)
  */
 #define IP_VS_APP_MAX_PORTS  8
 extern int register_ip_vs_app(struct ip_vs_app *mapp,
-                              unsigned short proto, __u16 port);
+			      unsigned short proto, __u16 port);
 extern int unregister_ip_vs_app(struct ip_vs_app *mapp);
 extern struct ip_vs_app * ip_vs_bind_app(struct ip_vs_conn *cp);
 extern int ip_vs_unbind_app(struct ip_vs_conn *cp);
 extern int ip_vs_app_pkt_out(struct ip_vs_conn *, struct sk_buff **skb_p);
 extern int ip_vs_app_pkt_in(struct ip_vs_conn *, struct sk_buff **skb_p);
 extern struct sk_buff * ip_vs_skb_replace (struct sk_buff *skb, int pri,
-                                           char *o_buf, int o_len,
-                                           char *n_buf, int n_len);
+					   char *o_buf, int o_len,
+					   char *n_buf, int n_len);
 extern int ip_vs_app_init(void);
 extern void ip_vs_app_cleanup(void);
 
@@ -526,8 +534,10 @@ extern void ip_vs_app_cleanup(void);
  */
 extern int register_ip_vs_scheduler(struct ip_vs_scheduler *scheduler);
 extern int unregister_ip_vs_scheduler(struct ip_vs_scheduler *scheduler);
+extern int ip_vs_register_scheduler_module(struct ip_vs_scheduler *scheduler);
+extern void ip_vs_unregister_scheduler_module(struct ip_vs_scheduler *scheduler);
 extern int ip_vs_bind_scheduler(struct ip_vs_service *svc,
-                                struct ip_vs_scheduler *scheduler);
+				struct ip_vs_scheduler *scheduler);
 extern int ip_vs_unbind_scheduler(struct ip_vs_service *svc);
 extern struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name);
 
@@ -536,17 +546,18 @@ extern struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name);
  *      IPVS control data and functions
  *      (from ip_vs_ctl.c)
  */
+extern int sysctl_use_dst;
 extern int sysctl_ip_vs_cache_bypass;
 extern atomic_t ip_vs_dropentry;
 extern struct ip_vs_stats ip_vs_stats;
 extern rwlock_t __ip_vs_svc_lock;
 
 extern struct ip_vs_service *ip_vs_service_get(__u32 fwmark,
-                                               __u16 protocol,
-                                               __u32 vaddr, __u16 vport);
+					       __u16 protocol,
+					       __u32 vaddr, __u16 vport);
 static inline void ip_vs_service_put(struct ip_vs_service *svc)
 {
-        atomic_dec(&svc->refcnt);
+	atomic_dec(&svc->usecnt);
 }
 
 extern struct ip_vs_dest *
@@ -592,28 +603,17 @@ extern void ip_vs_sltimer_cleanup(void);
 
 extern __inline__ char ip_vs_fwd_tag(struct ip_vs_conn *cp)
 {
-        char fwd = 'M';
+	char fwd = 'M';
 
-        switch (IP_VS_FWD_METHOD(cp)) {
-        case IP_VS_CONN_F_LOCALNODE:
-                fwd = 'L'; break;
-        case IP_VS_CONN_F_TUNNEL:
-                fwd = 'T'; break;
-        case IP_VS_CONN_F_DROUTE:
-                fwd = 'R'; break;
-        }
-        return fwd;
-}
-
-
-/*
- *	csum_partial wrapper, because csum_partial/i386 cannot provide
- *	correct result when len=0 and buff isn't double word aligned.
- */
-extern inline unsigned int
-ip_vs_csum_partial(const unsigned char * buff, int len, unsigned int sum)
-{
-	return len ? csum_partial(buff, len, sum) : sum;
+	switch (IP_VS_FWD_METHOD(cp)) {
+	case IP_VS_CONN_F_LOCALNODE:
+		fwd = 'L'; break;
+	case IP_VS_CONN_F_TUNNEL:
+		fwd = 'T'; break;
+	case IP_VS_CONN_F_DROUTE:
+		fwd = 'R'; break;
+	}
+	return fwd;
 }
 
 
@@ -644,43 +644,20 @@ __ip_vs_dst_reset(struct ip_vs_dest *dest)
 static inline struct dst_entry *
 __ip_vs_dst_check(struct ip_vs_dest *dest, u32 rtos, u32 cookie)
 {
-        struct dst_entry *dst = dest->dst_cache;
+	struct dst_entry *dst = dest->dst_cache;
 
-        if (!dst)
-                return NULL;
-        if ((dst->obsolete || rtos != dest->dst_rtos) &&
-            dst->ops->check(dst, cookie) == NULL) {
-                dest->dst_cache = 0;
-                return NULL;
-        }
-        dst_hold(dst);
-        return dst;
-}
-
-/*
- *	Own skb_cow() beast, tweaked for rewriting commonly
- *	used pointers in the code
- */
-static inline struct sk_buff *vs_skb_cow(struct sk_buff **skb_p,
-                                  struct iphdr **iph_p, unsigned char **t_p)
-{
-	struct sk_buff *skb=(*skb_p);
-	if (skb_cloned(skb)) {
-		skb = skb_copy(skb, GFP_ATOMIC);
-		if (skb) {
-			/*
-			 *	skb changed, update other pointers
-			 */
-			struct iphdr *iph = skb->nh.iph;
-			kfree_skb(*skb_p);
-			*skb_p = skb;
-			*iph_p = iph;
-			*t_p = (char*) iph + iph->ihl * 4;
-		}
+	if (!dst)
+		return NULL;
+	if ((dst->obsolete || rtos != dest->dst_rtos) &&
+	    dst->ops->check(dst, cookie) == NULL) {
+		dest->dst_cache = 0;
+		return NULL;
 	}
-	return skb;
+	dst_hold(dst);
+	return dst;
 }
 
+
 #endif /* __KERNEL__ */
 
 #endif	/* _IP_VS_H */
